<?php    
    $shortopts = "i:m:p:o:r:t:n:u:f";
    $options = getopt($shortopts);
 
    if (!isset($options['i'])) {
            showHelp();
    exit(0);
    } else {			
            $in = $options['i'];
            
            if (!isset($options['m'])) {
                    $media = pathinfo($in, PATHINFO_DIRNAME);
                    echo "No Media Directory specified, using ".realpath($media)."\n";
            } else {
            	$media = $options['m'];
            }

            if (isset($options['p'])) {
                    $info['projectName'] = $options['p'];
            }

            if (!isset($options['o'])) {
                    $out = realpath(pathinfo($in, PATHINFO_DIRNAME))."/".pathinfo($in, PATHINFO_FILENAME).".xml";
                    echo "No Output specified, using ".$out."\n";
            } else {
            	$out = $options['o'];
            }

            if (isset($options['r'])) {
                    $info['reelName'] = $options['r'];
            }

            if (isset($options['t'])) {
                    $info['taskName'] = $options['t'];
            }
            
            if (isset($options['n'])) {
                    $info['nameColumn'] = $options['n'];
            }
            else 
            {
            	if (isset($options['u']))
            	{
            		$info['nameColumnNoPath'] = $options['u']; 
            	}
            }

            $copraImport = aleToCopraImport($in, $media, $info, isset($options['f']));

            convertToXml($copraImport, $out);
    }
        
function showHelp() {
    echo "DaVinci Resolve to COPRA4 XML Converter:\n\n";
    echo "Usage: php copraxmlconvert.php -i <input> -o <output> -m <media> -p <project> -t <task> -r <reel> -n <file_header> -u <file_header_rename> -f\n\n";
    echo "<input>       Input ALE file created using DaVinci Resolve [Required]\n";
    echo "<output>      Path to COPRA4 output file [Optional]\n";
    echo "<media>       Path to Directory in which the media files are stored. [Optional]\n";
    echo "<project>     Targetted Project Name on COPRA4 Server [Optional]\n";
    echo "<task>        Targetted Task Name on COPRA4 Server [Optional]\n";
    echo "<reel>        Targetted Reel Name on COPRA4 Server [Optional]\n";
    echo "<file_header> Custom ALE Header name which includes the filename [Optional]\n";
    echo "<file_header_rename> Custom ALE Header name which includes the filename, filepath will be ignored[Optional]\n";
    echo "-f            Force: Create XML file even if Video files can not be found [Optional]\n\n";
}	

function aleToCopraImport($input, $mediaDirectory = "./", $info = NULL, $force = false) {
    // No File Given?
    if (strlen($input) < 1)
        exit(-1);
        
    if (!isset($info['projectName'])) {
        $proj = "COPRA4 ALE Import";
        $aleProjectName = true;
    } else {
        $proj = $info['projectName'];
    }
    
    if (!isset($info['taskName'])) {
        $task = "DAILIES";
    } else {
        $task = $info['taskName'];
    }
    
    if (!isset($info['reelName'])) {
        /* Generating Reel Name from File Name */
        $path = explode("/", $input);
        $reelName = substr($path[count($path)-1], 0, strlen($path[count($path)-1])-4);
    } else {
        $reelName = $info['reelName'];
    }
    
    $export['project']['name'] = $proj;
    $export['project']['taskType'] = "1";
    $export['project']['taskName'] = $task;
    $export['project']['reelName'] = $reelName;
    
    /* Need to check where in the File Data + Heading Columns begin for proper conversion */
    $file = file($input);
    
    $validHeadings = array("name", "start", "camera", "camroll", "fps", "labroll", 
        "reel #", "scene", "shoot day", "take", "tape", "comments", "unc", "shot", 
        "manufacturer", "camera type", "colorist", "production company", "dop", "editor", 
        "production name", "producer", "circled take", "director");
    
    $bNoPath = 0;
    if (isset($info['nameColumn'])) {
        $validHeadings[] = strtolower($info['nameColumn']);
        $customNameColumn = strtolower($info['nameColumn']);
    }
    else if (isset($info['nameColumnNoPath']))
    {
    	$validHeadings[] = strtolower($info['nameColumnNoPath']);
    	$customNameColumn = strtolower($info['nameColumnNoPath']);
    	$bNoPath = 1;
    }
        
    $headerRow = false;
    $added = 0;
    foreach($file as $line => $line_number) {
        if ($line === $headerRow) {
            echo "Preparing ALE Headers\n";
            $headings = strtolower($file[$headerRow]);
            $headings = explode("\t", $headings);
            
            $j = 0;
            echo "Valid Headers Found:";
            foreach ($headings as $header) {
                $res = array_search($header, $validHeadings);
        
                if ($res !== false) {
                    $copraHeaders[$header] = $j;
                    echo " ".$header;
                }
                $j++;
            }
            echo "\n";
            
            if (isset($customNameColumn) && !isset($copraHeaders[$customNameColumn])) {
                echo "Could not find '".$customNameColumn."' column to use as filename.\n";
            } else {
                echo "Using '".$customNameColumn."' column as filename.\n";
            }
            
            continue;
        }
        
        
        $str = $file[$line];
        
        if ($line >= $dataBegin && !empty($dataBegin) && $line < count($file)) {
            echo ($added+1)."... ";
            
            $column = explode("\t", $str);
            
            if ($mediaDirectory[strlen($mediaDirectory)-1] != "/")
                $mediaDirectory = $mediaDirectory."/";
            
            $filePath = "";
            
            $f_orig_name = $column[$copraHeaders['name']];
            $f_base_name = pathinfo($column[$copraHeaders['name']], PATHINFO_FILENAME);
            if (isset($customNameColumn) && $bNoPath == 0) {
                if (strlen($column[$copraHeaders[$customNameColumn]]) > 0) {
                    $f_orig_name = $column[$copraHeaders[$customNameColumn]];
                    $f_base_name = pathinfo($column[$copraHeaders[$customNameColumn]], PATHINFO_FILENAME);
                }
            }
            
            if (file_exists($mediaDirectory.$f_orig_name)) {
                $filePath = realpath($mediaDirectory.$f_orig_name);
            } else if (file_exists(realpath(pathinfo($input, PATHINFO_DIRNAME)."/".$f_orig_name))) {
                $filePath = realpath(pathinfo($input, PATHINFO_DIRNAME)."/".$f_orig_name);
            } else {           
                $extensions = array(".mov", ".m4v");
                
                foreach ($extensions as $ext) {
                
                    if (file_exists($mediaDirectory.$f_base_name.$ext)) {
                        $filePath = realpath($mediaDirectory.$f_base_name.$ext);
                        break;
                    }
                    
                    if (file_exists(realpath(pathinfo($input, PATHINFO_DIRNAME)."/".$f_base_name.$ext))) {
                        $filePath = realpath(pathinfo($input, PATHINFO_DIRNAME)."/".$f_base_name.$ext);
                        break;
                    }                
                }
            }

            if (strlen($filePath) < 1) {
            	if (!$force) {
					echo "ERROR: No valid file path found!\n";
                	continue;
            	} else {
            		$filePath = realpath($mediaDirectory);
                        $filePath = $filePath."/".$f_orig_name.'.mov';
                        echo "FORCE: ".$filePath."\n";
            	}
            } else {
            	$md5sum = md5_file($filePath);
            	$size = filesize($filePath);
            }
            
            if (isset($column[$copraHeaders['tape']])) {
                $camr = $column[$copraHeaders['tape']];
            }
            
            if (isset($column[$copraHeaders['camroll']])) {
                $camr = $column[$copraHeaders['camroll']];
            }
            
            $export[$added]['filename'] = $column[$copraHeaders['name']];
            if (isset($customNameColumn)) {
                if (strlen($column[$copraHeaders[$customNameColumn]]) > 0) {
                    $export[$added]['filename'] = $column[$copraHeaders[$customNameColumn]];
                }
            }
                           
            $export[$added]['tc_start'] = $column[$copraHeaders['start']];
            $export[$added]['camera'] = $column[$copraHeaders['camera']];
            $export[$added]['camroll'] = $camr;
            $export[$added]['framerate'] = $column[$copraHeaders['fps']];
            $export[$added]['reelnr'] = $column[$copraHeaders['reel #']];
            $export[$added]['labroll'] = $column[$copraHeaders['labroll']];
            $export[$added]['scene_name'] = $column[$copraHeaders['scene']];
            $export[$added]['shot'] = $column[$copraHeaders['shot']];
            $export[$added]['shot_date'] = $column[$copraHeaders['shoot day']];
            $take = $column[$copraHeaders['take']];
            if (strlen($take) < 1)
                $take = "1";
            $export[$added]['take'] = $take;
            $export[$added]['tape'] = $column[$copraHeaders['tape']];
            $export[$added]['comment'] = $column[$copraHeaders['comments']];
            $export[$added]['preview_file'] = $filePath;
            
            if (isset($size))
            	$export[$added]['preview_size'] = $size;
            	
            if (isset($md5sum))
            	$export[$added]['preview_md5hash'] = $md5sum;
            	
            $export[$added]['transfer_operator'] = $column[$copraHeaders['colorist']];
            $export[$added]['transfer_system'] = $column[$copraHeaders['manufacturer']];
            
            if ($column[$copraHeaders['circled take']] == 1)
                $export[$added]['selectedTake'] = 1;
            
            $comp = $column[$copraHeaders['production company']];
            if (empty($companies)) {
            	$companies[] = $comp;
            } else if (array_search($comp, $companies) === false) {
                $companies[] = $comp;
            }
            
            $producer = $column[$copraHeaders['producer']];
            if (empty($producers)) {
            	$producers[] = $producer;
            } else if (array_search($producer, $producers) === false) {
                $producers[] = $producer;
            }
            
            $camera = $column[$copraHeaders['camera type']];
            if (empty($cameras)) {
            	$cameras[] = $camera;
            } else if (array_search($camera, $cameras) === false) {
                $cameras[] = $camera;
            }
            
            $dop = $column[$copraHeaders['dop']];
            if (empty($dops)) {
            	$dops[] = $dop;
            } else if (array_search($dop, $dops) === false) {
                $dops[] = $dop;
            }
            
            $editor = $column[$copraHeaders['editor']];
            if (empty($editors)) {
            	$editors[] = $editor;
            } else if (array_search($editor, $editors) === false) {
                $editors[] = $editor;
            }
            
            $director = $column[$copraHeaders['director']];
            if (empty($directors)) {
            	$directors[] = $director;
            } else if (array_search($director, $directors) === false) {
                $directors[] = $director;
            }
            
            if (strlen($column[$copraHeaders['production name']]) > 0)
                $productionName = $column[$copraHeaders['production name']];
            
            echo "Success!\n";
            $added++;
            continue;
        }
        
        if (strpos($str, "Column") !== false)
            $headerRow = $line+1;
        
        if (strpos($str, "Data") !== false) {
            echo "Parsing Clips...\n";
            $dataBegin = $line+1;
        }
    }
    
    if ($added < 1)
        echo "No clips have been parsed. No array to return.\n";
    else
        echo "Parsing successful.\n";
    
    if (count($companies) > 0)
        $export['project']['production'] = $companies;
    
    if (count($producers) > 0)
        $export['project']['producer'] = $producers;
    
    if (count($cameras) > 0)
        $export['project']['materials'] = $cameras;
    
    if (count($dops) > 0)
        $export['project']['dop'] = $dops;
    
    if (count($editors) > 0)
        $export['project']['editing'] = $editors;
        
    if (count($directors) > 0)
        $export['project']['director'] = $directors;
    
    if (strlen($productionName) > 0 && $aleProjectName == true)
        $export['project']['name'] = $productionName;
    
    return $export;
}

function stringFromArrayOrString($input) {
    if (count($input) > 0) {
        $out = "";
        for ($i = 0; $i < count($input); $i++) {
            if ($i != 0)
                $out = $out.", ";
            $out = $out.$input[$i];
        }        
        return $out;
    } 
    
    return $input;
}

function convertToXml($copraImport, $filename = "temp.xml") {
    if (count($copraImport) < 1) {
        echo "ERROR: Empty array. Not processing.\n";
        return "-1";
    }
    
    echo "Processing Project Settings.\n";
    
    $projectSettings = $copraImport['project'];
    
    if (strlen($projectSettings['taskName']) < 1 || strlen($projectSettings['reelName']) < 1 || strlen($projectSettings['name']) < 1) {
        echo "ERROR: Task Name, Reel Name or Project Name are empty.\n";
        return "-1";
    }
    
    $clipData = $copraImport;
    
    echo "Creating XML Structure...\n";
    
    $dom = new DOMDocument("1.0", "UTF-8");    
    $dom_root = $dom->createElement("copraimport");
    $dom->appendChild($dom_root);
    $dom_xmlversion = $dom->createAttribute("xmlversion");
    $dom_root->appendChild($dom_xmlversion);
    $dom_item = $dom->createTextNode("1.0");
    $dom_xmlversion->appendChild($dom_item);
    
    echo "Processing Project Data\n";
    if (strlen($projectSettings['token']) > 0) {
        $dom_token = $dom->createElement("token", $projectSettings['token']);
        $dom_root->appendChild($dom_token);
    }
    
    $dom_project = $dom->createElement("project");
    $dom_root->appendChild($dom_project);
        
    $dom_pname = $dom->createAttribute("name");
    $dom_project->appendChild($dom_pname);
    $dom_item = $dom->createTextNode($projectSettings['name']);
    $dom_pname->appendChild($dom_item);
    
    $projectSubNodes = array("production", "producer", "director", "dop", "editing", "shooting", "locations", "materials", "dailies", "thumbnail");
    
    foreach ($projectSubNodes as $subnode) {
        $str = stringFromArrayOrString($projectSettings[$subnode]);
        if (strlen($str) > 0) {
            $dom_item = $dom->createElement($subnode, $str);
            $dom_project->appendChild($dom_item);
        }
    }
    
    echo "Project Data Generated\nProcessing Task + Reel Data...\n";
    
    $dom_tasks = $dom->createElement("tasks");
    $dom_project->appendChild($dom_tasks);
    $dom_task = $dom->createElement("task");
    
    if (strlen($projectSettings['task_thumbnail']) > 0) {
        $dom_taskThumb = $dom->createElement("thumbnail", $projectSettings['task_thumbnail']);
        $dom_task->appendChild($dom_taskThumb);
    }
    
    $dom_tasks->appendChild($dom_task);
    $dom_tname = $dom->createAttribute("name");
    $dom_task->appendChild($dom_tname);
    $dom_item = $dom->createTextNode($projectSettings['taskName']);
    $dom_tname->appendChild($dom_item);
    
    $dom_ttype = $dom->createAttribute("type");
    $dom_task->appendChild($dom_ttype);
    $dom_item = $dom->createTextNode($projectSettings['taskType']);
    $dom_ttype->appendChild($dom_item);
    $dom_reels = $dom->createElement("reels");
    $dom_task->appendChild($dom_reels);
    $dom_reel = $dom->createElement("reel");
    $dom_reels->appendChild($dom_reel);
    $dom_rname = $dom->createAttribute("name");
    $dom_reel->appendChild($dom_rname);
    
    $dom_item = $dom->createTextNode($projectSettings['reelName']);
    $dom_rname->appendChild($dom_item);
    
    if (strlen($projectSettings['task_thumbnail']) > 0) {
        $dom_reelthumb = $dom->createElement("thumbnail", $projectSettings['reel_thumbnail']);
        $dom_reel->appendChild($dom_reelthumb);
    }
    
    $dom_takes = $dom->createElement("takes");
    $dom_reel->appendChild($dom_takes);
    
    $addedTakes = 0;
    
    echo "Task + Reel Data generated\nProcessing Takes...\n";
    
    for ($x = 0; $x < count($clipData)-1; $x++) {
        $clip = $clipData[$x];
        
        echo ($x+1)."... ";
        
        if (strlen($clip['take']) < 1) {
            echo "ERROR: Take Name is not set. Skipping\n";
            continue;
        }
        
        if (strlen($clip['preview_file']) < 1) {
            echo "ERROR: Take path is not set. Skipping\n";
            continue;
        }
                
        $addedTakes++;
        
        $dom_take = $dom->createElement("take");
        $dom_takes->appendChild($dom_take);
        $dom_tname = $dom->createAttribute("name");
        $dom_take->appendChild($dom_tname);
        $dom_item = $dom->createTextNode($clip['take']);
        $dom_tname->appendChild($dom_item);
        
        $dom_sname = $dom->createAttribute("scene_name");
        $dom_take->appendChild($dom_sname);
        $dom_item = $dom->createTextNode($clip['scene_name']);
        $dom_sname->appendChild($dom_item);
        
        $takeSubNodes = array("filename", "camroll", "camera", "framerate", "tc_start", "shot", "shot_date", "frames", 
            "unique_id", "clipname", "source_format", "source_width", "source_height", "playrate", "sound_sync", 
            "rating", "preview_bitrate", "preview_size", "thumbnail", "preview_file", "preview_md5hash");
        
        for ($i = 0; $i < count($takeSubNodes); $i++) {
            $t_subnode = $takeSubNodes[$i];
            if (strlen($clip[$t_subnode]) > 0) {
                $dom_node = $dom->createElement($t_subnode);
                $dom_take->appendChild($dom_node);
                $dom_item = $dom->createTextNode($clip[$t_subnode]);
                $dom_node->appendChild($dom_item);
            }
        }
        
        if (strlen($clip['transfer_system']) > 0 || strlen($clip['transfer_operator']) > 0) {
            $dom_transfer = $dom->CreateElement("transfer");
            $dom_take->appendChild($dom_transfer);
            
            if (strlen($clip['transfer_system']) > 0) {
                $dom_t_system = $dom->CreateElement("system");
                $dom_transfer->appendChild($dom_t_system);
                $dom_item = $dom->CreateTextNode($clip['transfer_system']);
                $dom_t_system->appendChild($dom_item);
            }

            if (strlen($clip['transfer_operator']) > 0) {
                $dom_t_operator = $dom->CreateElement("operator");
                $dom_transfer->appendChild($dom_t_operator);
                $dom_t_opname = $dom->CreateElement("name");
                $dom_t_operator->appendChild($dom_t_opname);
                $dom_item = $dom->CreateTextNode($clip['transfer_operator']);
                $dom_t_opname->appendChild($dom_item);
            }
        }
        
        if (strlen($clip['comment']) > 0) {
            $dom_notes = $dom->CreateElement("notes");
            $dom_note = $dom->CreateElement("note");
            $dom_notes->appendChild($dom_note);
            $dom_qcid = $dom->createAttribute("qc_number");
            $dom_item = $dom->createTextNode("1");
            $dom_qcid->appendChild($dom_item);
            $dom_note->appendChild($dom_qcid);
            
            $dom_note_text = $dom->createElement("text");
            $dom_item = $dom->createTextNode($clip['comment']);
            $dom_note_text->appendChild($dom_item);
            $dom_note->appendChild($dom_note_text);
            
            if (strlen($clip['qc_tc_start']) > 0) {
                $dom_note_starttc = $dom->createElement("tc_start");
                $dom_item = $dom->createTextNode($clip['qc_tc_start']);
                $dom_note_starttc->appendChild($dom_item);
                $dom_note->appendChild($dom_note_starttc);
            }
            
            if (strlen($clip['qc_tc_end']) > 0) {
                $dom_note_endtc = $dom->createElement("tc_end");
                $dom_item = $dom->createTextNode($clip['qc_tc_end']);
                $dom_note_endtc->appendChild($dom_item);
                $dom_note->appendChild($dom_note_endtc);
            }
            
            $dom_take->appendChild($dom_notes); 
        }
        
        echo "Success!\n";
    }
    
    if ($addedTakes < 1) {
        echo "No Takes have been added.\n";
        return "-1";
    }
    
    echo "COPRA4 Import XML Generation Successful.\nSaving to Path: ", $filename."\n";
    $dom->formatOutput = TRUE; // this adds spaces, new lines and makes the XML more readable format.
    $dom->save($filename);
}

?>
